/* $Id: Ip2SourceMapper.c,v 1.27 2011/09/15 21:55:56 ruiliu Exp $ */

/****************************************************************************
 *
 *                      University of Illinois/NCSA
 *                          Open Source License
 * 
 *          Copyright(C) 2009-2011, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                             Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * JNI implementation of source code mapping using libpsx.
 *
 * Author:        Rui Liu
 * Date Started:  3/2009
 *
 * This file is part of PerfSuite.
 *
 */

#include <perfsuite.h>

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#if HAVE_JNI_H
#  include <jni.h>
#endif

#if HAVE_STDLIB_H
#  include <stdlib.h>
#endif

/* for timing */
#if HAVE_SYS_TIME_H
#  include <sys/time.h>
#endif

#if HAVE_TIME_H
#  include <time.h>
#endif

#include <psx.h>


#define PRINT_ELAPSED_TIME(X,Y)                                 \
    if (debug_level >= VERBOSE) {                               \
        gettimeofday (&t0, NULL);                               \
    }                                                           \
    Y;                                                          \
    if (debug_level >= VERBOSE) {                               \
        gettimeofday (&t1, NULL);                               \
        printf ("elapsed time for '%s': %f ms.\n",              \
                X,                                              \
                ((t1.tv_usec - t0.tv_usec) / 1.0e6 +            \
                 (t1.tv_sec - t0.tv_sec)) * 1.0e3);             \
    }


#define ERROR(X,Y)                                                      \
    if (debug_level >= OFF) {                                           \
        printf("error in file %s, at line %d, in calling %s, "          \
               "ret code is %d.\n", __FILE__, __LINE__, X, Y);          \
    }

/* Took advantage of the fact that ip and pmap are NULL before they
   are defined, and free (NULL) is no-op, so no harm. Use this to
   simplify error handling, to un-clutter code.
   This macro can only be used after a successful call to
   GetLongArrayElements(...), since it calls the Release... function.
*/
#define RELEASE_RESOURCE_RETURN                                         \
    (*env)->ReleaseLongArrayElements (env, ips, pips, 0);               \
                                                                        \
    free (ip);                                                          \
    ip = NULL;                                                          \
                                                                        \
    free (pmap);                                                        \
    pmap = NULL;                                                        \
    return;

#define ERROR_RETURN(X,Y)                                               \
    ERROR(X,Y)                                                          \
    RELEASE_RESOURCE_RETURN

#define CHECK_NULL_ERROR_RETURN(Z,X,Y)                                  \
    if (NULL == Z) {                                                    \
        ERROR_RETURN(X,Y);                                              \
    }

#define CHECK_NULL_REL_STR_RETURN_VALUE(Z,X,Y)                          \
    if (NULL == Z) {                                                    \
        (*env)->ReleaseStringUTFChars (env, binaryFileName, filename);  \
        ERROR(X,Y);                                                     \
        return ERROR_BFD_JNI;                                           \
     }

#define CHECK_NULL_ERROR_RETURN_VALUE(Z,X,Y)                            \
    if (NULL == Z) {                                                    \
        ERROR(X,Y);                                                     \
        return ERROR_BFD_JNI;                                           \
     }

static int debug_level = 0;
static const int OFF     = 0;
/*  They are not used for now, so commented them out. */
/*
static const int FATAL   = 1;
static const int WARNING = 2;
static const int INFO    = 3;
*/
static const int VERBOSE = 4;

/* function prototypes */

BEGIN_C_DECLS

JNIEXPORT jint JNICALL Java_org_perfsuite_util_Ip2SourceMapper_jniinit
PARAMS((JNIEnv *, jclass));
JNIEXPORT jint JNICALL Java_org_perfsuite_util_Ip2SourceMapper_jniopen
PARAMS((JNIEnv *, jobject, jstring));
JNIEXPORT jint JNICALL Java_org_perfsuite_util_Ip2SourceMapper_jniclose
PARAMS((JNIEnv *, jobject));
JNIEXPORT void JNICALL Java_org_perfsuite_util_Ip2SourceMapper_jnimap
PARAMS((JNIEnv *, jobject, jlongArray, jlong));
JNIEXPORT jstring JNICALL Java_org_perfsuite_util_Ip2SourceMapper_jnidemangle
PARAMS((JNIEnv *, jclass, jstring, jstring));
JNIEXPORT jboolean JNICALL Java_org_perfsuite_util_Ip2SourceMapper_jni_1have_1cplus_1demangle
PARAMS((JNIEnv *, jobject));

END_C_DECLS

/*
 * Class:     org_perfsuite_util_Ip2SourceMapper
 * Method:    jniinit
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_org_perfsuite_util_Ip2SourceMapper_jniinit
(JNIEnv * env, jclass cls)
{
    jint ret = _psx_bfd_init ();
    const char * debugEnv = (const char *) getenv ("PS_DEBUG");
    if (debugEnv != NULL) {
        int level = atoi (debugEnv);
        if (level > 0)
            debug_level = level;
    }
#if SIZEOF_VOID_P != 4 && SIZEOF_VOID_P != 8
# error "This architecture is not supported because sizeof(void *) != 4 and sizeof(void *) != 8"
#endif
    return ret;
}


/*
 * Class:     org_perfsuite_util_Ip2SourceMapper
 * Method:    jniopen
 * Signature: (Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_org_perfsuite_util_Ip2SourceMapper_jniopen
(JNIEnv * env, jobject obj, jstring binaryFileName)
{
    jint ret = 0;
    const char *filename = NULL;
    ps_bfd_t bfd = NULL;
    jclass cls = NULL;
    jfieldID idbfd = NULL;
    jint ERROR_BFD_JNI = -3;
    struct timeval t0;
    struct timeval t1;
#if SIZEOF_VOID_P != 8
    int bfdint = 0;
#endif

    filename = (*env)->GetStringUTFChars (env, binaryFileName, 0);
    if (NULL == filename) {
        ERROR("GetStringUTFChars",0);
        return ERROR_BFD_JNI;
    }
    /* any return below this needs to "release filename" beforehand */

    PRINT_ELAPSED_TIME("_psx_bfd_open ",ret = _psx_bfd_open ((char *)filename, &bfd););

    if (ret != PS_SUCCESS) {
        (*env)->ReleaseStringUTFChars (env, binaryFileName, filename);
        ERROR("_psx_bfd_open",ret);
        return ret;
    }

    cls = (*env)->GetObjectClass (env, obj);
    CHECK_NULL_REL_STR_RETURN_VALUE(cls,"GetObjectClass",0);
    idbfd = (*env)->GetFieldID (env, cls, "bfd", "J");
    CHECK_NULL_REL_STR_RETURN_VALUE(idbfd,"GetFieldID for idbfd",0);

#if SIZEOF_VOID_P == 8
    (*env)->SetLongField (env, obj, idbfd, (jlong) bfd);
#else
    bfdint = (int) bfd;
    (*env)->SetLongField (env, obj, idbfd, (jlong) bfdint);
#endif

    if (debug_level >= VERBOSE) {
        printf ("in jniopen(), bfd = %p\n", bfd);
    }

    (*env)->ReleaseStringUTFChars (env, binaryFileName, filename);
    return PS_SUCCESS;
}

/*
 * Class:     org_perfsuite_util_Ip2SourceMapper
 * Method:    jniclose
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_org_perfsuite_util_Ip2SourceMapper_jniclose
(JNIEnv * env, jobject obj)
{
    jint ret = 0;
    jclass cls = NULL;
    jfieldID idbfd = NULL;
    jlong bfd = 0;
    jint ERROR_BFD_JNI = -1;
    struct timeval t0;
    struct timeval t1;
#if SIZEOF_VOID_P != 8
    int bfdint = 0;
#endif

    cls = (*env)->GetObjectClass (env, obj);
    CHECK_NULL_ERROR_RETURN_VALUE(cls,"GetObjectClass",0);
    idbfd = (*env)->GetFieldID (env, cls, "bfd", "J");
    CHECK_NULL_ERROR_RETURN_VALUE(idbfd,"GetFieldID for idbfd",0);

    bfd = (*env)->GetLongField (env, obj, idbfd);

    if (debug_level >= VERBOSE) {
        printf ("in jniclose(), bfd = %#llx.\n", (unsigned long long) bfd);
    }

#if SIZEOF_VOID_P == 8
    PRINT_ELAPSED_TIME("_psx_bfd_close",ret = _psx_bfd_close ((ps_bfd_t) bfd););
#else
    bfdint = (int) bfd;
    PRINT_ELAPSED_TIME("_psx_bfd_close",ret = _psx_bfd_close ((ps_bfd_t) bfdint););
#endif

    if (ret != PS_SUCCESS) {
        ERROR("_psx_bfd_close",ret);
    }

    return ret;
}

/*
 * Class:     org_perfsuite_util_Ip2SourceMapper
 * Method:    jnimap
 * Signature: ([JJ)V
 */
JNIEXPORT void JNICALL Java_org_perfsuite_util_Ip2SourceMapper_jnimap
(JNIEnv * env, jobject mapperObj, jlongArray ips, jlong offset0)
{
    jint ret = 0;

    caddr_t *ip = NULL;
    ps_ip_map_t *pmap = NULL;
    ps_bfd_t bfd = NULL;
    int i;
    int count = (*env)->GetArrayLength(env, ips);
    jlong *pips = NULL;

    jclass cls = NULL;
    jfieldID idbfd = NULL;
    jfieldID idip = NULL;
    jfieldID idfile = NULL;
    jfieldID idfunction = NULL;
    jfieldID idline = NULL;

    jfieldID idresult = NULL;
    jobjectArray result = NULL;
    jlong offset = 0;

    struct timeval t0;
    struct timeval t1;

    int flagMatches = 0;
#if SIZEOF_VOID_P != 8
    int bfdint = 0;
    int tmpip = 0;
#endif

    if (count <= 0) {
        return;
    }

    cls = (*env)->GetObjectClass (env, mapperObj);
    if (NULL == cls) {
        ERROR("GetObjectClass",0);
        return;
    }

    idbfd = (*env)->GetFieldID (env, cls, "bfd", "J");
    if (NULL == idbfd) {
        ERROR("GetFieldID",0);
        return;
    }

#if SIZEOF_VOID_P == 8
    bfd = (ps_bfd_t) ((*env)->GetLongField (env, mapperObj, idbfd));
#else
    bfdint = (int) ((*env)->GetLongField (env, mapperObj, idbfd));
    bfd = (ps_bfd_t) bfdint;
#endif

    if (NULL == bfd) {
        ERROR("GetLongFieldID",0);
        return;
    }

    if (debug_level >= VERBOSE) {
        printf ("in jnimap(), bfd = %p\n", bfd);
    }
    if ( (PS_SUCCESS == _psx_bfd_inquire (bfd, PSX_BFD_FILE_FLAG_DYNAMIC,
                                          &flagMatches)) &&
         (1 == flagMatches) ) {
        offset = offset0;
    }

    pips = (*env)->GetLongArrayElements (env, ips, 0);
    if (NULL == pips) {
        ERROR("GetLongArrayElements",0);
        return;
    }
    /* any return below this needs to "release pips" beforehand */

    /* changed to always allocate ip, since in 3 out of 4 cases,
       ip needs to be allocated (64 bit so, 32 bit exe, 32 bit so).
       For simplicity, made the change. Rui 2009/08/21. */

    ip = (caddr_t *) malloc (count*sizeof(caddr_t));
    CHECK_NULL_ERROR_RETURN(ip,"malloc for ip",0);
    /* any return below this needs to "free ip" beforehand */

    /* The following assignment could generate compiler warning
       on x86 platform, such as:
           warning: cast to pointer from integer of different size
       Reason:
           The IP is 32 bit on x86, and could be 64 bit on ia64 or x86_64.
           To cover both cases, in Java "long" is used as the data type.
           However, _psx_bfd_map uses caddr_t, which is defined as "void *".
           On x86, this is 32 bit, so above compiler warning occurs.
           We can ignore it for now.
    */
    for (i = 0; i < count; i++) {

#if SIZEOF_VOID_P == 8
        ip[i] = (caddr_t) (pips[i] - offset);
#else
        tmpip = (int) (pips[i] - offset);
        ip[i] = (caddr_t) tmpip;
#endif

    }

    pmap = (ps_ip_map_t *) malloc (count*sizeof(ps_ip_map_t));
    CHECK_NULL_ERROR_RETURN(pmap,"malloc for pmap",0);
    /* any return below this needs to "free pmap" beforehand */

    cls = (*env)->FindClass (env, "org/perfsuite/util/Ip2SourceMapEntry");
    CHECK_NULL_ERROR_RETURN(cls,"FindClass",0);

    idip       = (*env)->GetFieldID (env, cls, "ip", "J");
    CHECK_NULL_ERROR_RETURN(idip,"GetFieldID for ip",0);
    idfile     = (*env)->GetFieldID (env, cls, "file", "Ljava/lang/String;");
    CHECK_NULL_ERROR_RETURN(idfile,"GetFieldID for file",0);
    idfunction = (*env)->GetFieldID (env, cls, "function",
                                     "Ljava/lang/String;");
    CHECK_NULL_ERROR_RETURN(idfunction,"GetFieldID for function",0);
    idline     = (*env)->GetFieldID (env, cls, "line", "I");
    CHECK_NULL_ERROR_RETURN(idline,"GetFieldID for line",0);

    /* Get the result array. */
    cls = (*env)->GetObjectClass (env, mapperObj);
    CHECK_NULL_ERROR_RETURN(cls,"GetObjectClass",0);
    idresult = (*env)->GetFieldID
        (env, cls, "result", "[Lorg/perfsuite/util/Ip2SourceMapEntry;");
    CHECK_NULL_ERROR_RETURN(idresult,"GetFieldID for idresult",0);
    result = (*env)->GetObjectField (env, mapperObj, idresult);
    CHECK_NULL_ERROR_RETURN(result,"GetObjectField for result",0);


    PRINT_ELAPSED_TIME("_psx_bfd_map  ",ret = _psx_bfd_map (bfd, ip, count, pmap););

    for (i = 0; i < count; i++) {
        jobject obj1 = (*env)->GetObjectArrayElement (env, result, i);
        if (obj1) {
            if (pmap[i].function) {
                /* Some JVM implementation crashes if NewStringUTF is
                   called with NULL, so make it defensive. */
                (*env)->SetObjectField
                    (env, obj1, idfunction,
                     (*env)->NewStringUTF (env, pmap[i].function));
            }
            if (pmap[i].file) {
                (*env)->SetObjectField
                    (env, obj1, idfile,
                     (*env)->NewStringUTF (env, pmap[i].file));
            }
            (*env)->SetIntField (env, obj1, idline, pmap[i].line);
        }
    }

    RELEASE_RESOURCE_RETURN;
}

/*
 * Class:     org_perfsuite_util_Ip2SourceMapper
 * Method:    jnidemangle
 * Signature: (Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
 */
/* Note:      Returns "null" if _psx_demangle fails. */
JNIEXPORT jstring JNICALL Java_org_perfsuite_util_Ip2SourceMapper_jnidemangle
(JNIEnv * env, jclass cls, jstring inStr, jstring styleStr)
{
    const char *funcName = (*env)->GetStringUTFChars (env, inStr, 0);
    const char *style = (*env)->GetStringUTFChars (env, styleStr, 0);
    char *demangledName;
    jstring ret = NULL;

    if ( _psx_demangle (funcName, style, &demangledName) == PS_SUCCESS
         && (demangledName != NULL) ) {
        ret = (*env)->NewStringUTF (env, demangledName);
    }

    (*env)->ReleaseStringUTFChars (env, inStr, funcName);
    (*env)->ReleaseStringUTFChars (env, styleStr, style);
    return ret;
}

/*
 * Class:     org_perfsuite_util_Ip2SourceMapper
 * Method:    jni_have_cplus_demangle
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_perfsuite_util_Ip2SourceMapper_jni_1have_1cplus_1demangle
(JNIEnv * env, jobject obj)
{

#if CPLUS_DEMANGLE_PIC
    return 1;
#else
    return 0;
#endif

}
