/* $Id: PS_BFD_Util.c,v 1.6 2010/04/28 21:14:41 ruiliu Exp $ */

/****************************************************************************
 *
 *                      University of Illinois/NCSA
 *                          Open Source License
 * 
 *          Copyright(C) 2009-2010, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                             Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * JNI implementation of various bfd utilities.
 *
 * Author:        Rui Liu
 * Date Started:  9/2009
 *
 * This file is part of PerfSuite.
 *
 */

#include <perfsuite.h>

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#if HAVE_JNI_H
#  include <jni.h>
#endif

#if HAVE_STDLIB_H
#  include <stdlib.h>
#endif

#include <psx.h>



#define ERROR(X,Y)                                                      \
    if (debug_level >= OFF) {                                           \
        printf("error in file %s, at line %d, in calling %s, "          \
               "ret code is %d.\n", __FILE__, __LINE__, X, Y);          \
    }


static int debug_level = 0;
static const int OFF     = 0;
/*  They are not used for now, so commented them out. */
/*
static const int FATAL   = 1;
static const int WARNING = 2;
static const int INFO    = 3;
*/
static const int VERBOSE = 4;

/* function prototypes */
BEGIN_C_DECLS

JNIEXPORT jint JNICALL Java_org_perfsuite_util_PS_1BFD_1Util_jniinit
PARAMS((JNIEnv *, jclass));
JNIEXPORT jint JNICALL Java_org_perfsuite_util_PS_1BFD_1Util_jniisDynamic
PARAMS((JNIEnv *, jclass, jstring));

END_C_DECLS

/*
 * Class:     org_perfsuite_util_PS_BFD_Util
 * Method:    jniinit
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_org_perfsuite_util_PS_1BFD_1Util_jniinit
(JNIEnv * env, jclass cls)
{
    jint ret = _psx_bfd_init ();
    const char * debugEnv = (const char *) getenv ("PS_DEBUG");
    if (debugEnv != NULL) {
        int level = atoi (debugEnv);
        if (level > 0)
            debug_level = level;
    }
    return ret;
}



/*
 * Class:     org_perfsuite_util_PS_BFD_Util
 * Method:    jniisDynamic
 * Signature: (Ljava/lang/String;)I
 */
/* Returns: 1 if is dynamic, 0 if not, -1 if can not determine. */
JNIEXPORT jint JNICALL Java_org_perfsuite_util_PS_1BFD_1Util_jniisDynamic
(JNIEnv * env, jclass cls, jstring binaryFileName)
{
    jint result = -1;
    int ret = 0;
    const char *filename = NULL;
    ps_bfd_t bfd = NULL;
    int flagMatches = 0;

    filename = (*env)->GetStringUTFChars (env, binaryFileName, 0);
    if (NULL == filename) {
        ERROR("GetStringUTFChars",0);
        return -1;
    }
    /* any return below this needs to "release filename" beforehand */

    ret = _psx_bfd_open ((char *)filename, &bfd);

    if (ret != PS_SUCCESS) {
        (*env)->ReleaseStringUTFChars (env, binaryFileName, filename);
        ERROR("_psx_bfd_open",ret);
        return -1;
    }

    if ( (PS_SUCCESS == _psx_bfd_inquire (bfd, PSX_BFD_FILE_FLAG_DYNAMIC,
                                          &flagMatches)) &&
         (1 == flagMatches) ) {
        result = 1;
    } else {
        result = 0;
    }

    ret = _psx_bfd_close (bfd);
    if (ret != PS_SUCCESS) {
        ERROR("_psx_bfd_close",ret);
    }

    (*env)->ReleaseStringUTFChars (env, binaryFileName, filename);
    return result;
}
