/* $Id: PapiEvent.c,v 1.7 2010/01/29 20:11:16 ruiliu Exp $ */

/****************************************************************************
 *
 *                      University of Illinois/NCSA
 *                          Open Source License
 * 
 *            Copyright(C) 2009, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                             Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * JNI implementation of mapping a PAPI event name to its description,
 * using a PAPI library.
 *
 * Author:        Rui Liu
 * Date Started:  3/2009
 *
 * This file is part of PerfSuite.
 *
 */

#include <perfsuite.h>

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#if HAVE_JNI_H
#include <jni.h>
#endif

#if HAVE_PAPI_H
#include <papi.h>
#endif

BEGIN_C_DECLS

JNIEXPORT jboolean JNICALL Java_org_perfsuite_util_PapiEvent_initPAPI
PARAMS((JNIEnv *, jclass));
JNIEXPORT jstring JNICALL Java_org_perfsuite_util_PapiEvent_getEventDesc
PARAMS((JNIEnv *, jclass, jstring));

int getPapiEventInfo PARAMS((const char * eventName, PAPI_event_info_t * info));

END_C_DECLS

/*
 * Class:     org_perfsuite_util_PapiEvent
 * Method:    initPAPI
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_org_perfsuite_util_PapiEvent_initPAPI
(JNIEnv * env, jclass cls)
{
    return ( PAPI_library_init(PAPI_VER_CURRENT) == PAPI_VER_CURRENT );
}

/*
 * Class:     org_perfsuite_util_PapiEvent
 * Method:    getEventDesc
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_org_perfsuite_util_PapiEvent_getEventDesc
(JNIEnv * env, jclass cls, jstring javaString)
{
    const char *eventName = (*env)->GetStringUTFChars (env, javaString, 0);
    PAPI_event_info_t info;
    jstring ret = NULL;

    if ( getPapiEventInfo (eventName, &info) == PAPI_OK ) {
        ret = (*env)->NewStringUTF (env, info.long_descr);
    }

    (*env)->ReleaseStringUTFChars (env, javaString, eventName);
    return ret;
}

int getPapiEventInfo (const char * eventName, PAPI_event_info_t * info) {

    int ret = PAPI_OK;
    int eventCode = 0;

    ret = PAPI_event_name_to_code ( (char *) eventName, &eventCode);
    if (ret != PAPI_OK) {
        return ret;
    }

    return PAPI_get_event_info (eventCode, info);
}
