/* $Id: write-xml.h,v 1.3 2012/05/11 15:15:54 ruiliu Exp $ */

/****************************************************************************
 *
 *                      University of Illinois/NCSA
 *                          Open Source License
 * 
 *          Copyright(C) 2011-2012, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                             Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/* For writing out Java profiling XML files. */

#ifndef WRITE_XML_H
#define WRITE_XML_H 1

#include <stdio.h>
#include <cpuid.h>

#define PROFILEREPORTTAG "hwpcprofilereport"
#define PROFILEREPORT_DTDTYPE "hwpcprofilereport-0.3"
#define EXECINFOTAG      "executioninfo"

#ifndef PS_SUFFIX_XML
# define PS_SUFFIX_XML           "xml"
#endif

typedef struct {
    const char *module;
    const char *file;
    const char *function;
    int line;
    unsigned int num_samples;
} psjp_loc_t;

typedef struct {
    ps_cpuid_t cpuid;
    double mhz;
    const char * papi_version;
    int pid;
    char * main_class_name;
    char * main_method_args;
    char * event_domain;
} psjp_globals_t;

typedef struct {
    const char * event_name;
    const char * event_type;
    int          event_threshold;
    int          num_nodes;
    psjp_loc_t * locs;
    unsigned int num_total_samples;
} psjp_thread_event_profile_t;



BEGIN_C_DECLS

int
_ps_hwpc_xml_write_java PARAMS((FILE *fd, char *generator,
                                psjp_globals_t *globals,
                                unsigned int tid,
                                unsigned long long ticks,
                                double usersecs, double systemsecs,
                                int numEvents,
                                psjp_thread_event_profile_t * profiles,
                                int close, const char * threadName));

END_C_DECLS

#endif   /* #ifndef WRITE_XML_H */
