/* $Id: checksum.c,v 1.1 2006/09/30 15:55:53 rkufrin Exp $ */

/****************************************************************************
 *
 *                      University of Illinois/NCSA
 *                          Open Source License
 * 
 *            Copyright(C) 2006, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                             Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * checksum: generate MD5 checksums
 *
 * Author: Rick Kufrin
 * Date:   Sep 29, 2006
 *
 * This file is part of PerfSuite.
 */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#if _REENTRANT
# if HAVE_PTHREAD
#  include <pthread.h>
# endif
#endif

#include <stdio.h>

#if HAVE_STDLIB_H
# include <stdlib.h>
#endif

#include <perfsuite.h>
#include <psinternal.h>
#include "psdebug.h"
#include "constants.h"

#if HAVE_MD5_RFC
# include <md5/md5.h>
#endif

int
_ps_md5_file(char *fname, char md5out[PS_MD5LEN])
{
#if HAVE_MD5_RFC
    FILE *fin;
    md5_byte_t buf[BUFSIZ], digest[16];
    md5_state_t pms;
    int nbytes, di;

# if _REENTRANT
    pthread_cleanup_push((void *)_ps_pthread_cleanup_fd, &fin);
# endif

    fin = fopen(fname, "r");
    if ( fin == NULL ) {
        return PS_ESYS;
    }

    md5_init(&pms);

    for ( ;; ) {
        nbytes = fread(buf, sizeof(md5_byte_t), BUFSIZ, fin);
        if ( ferror(fin) ) {
            PS_DEBUG(PS_DEBUG_WARNING, \
                     fputs("MD5 checksum file read error\n", stderr);)
            fclose(fin);
            return PS_ESYS;
        }
        md5_append(&pms, buf, nbytes);
        if ( feof(fin) ) {
            break;
        }
    }

    fclose(fin);

    md5_finish(&pms, digest);

    for ( di = 0; di < 16; ++di ) {
        sprintf(md5out + di * 2, "%02x", digest[di]);
    }

# if _REENTRANT
    pthread_cleanup_pop(0);
# endif

    return PS_SUCCESS;
#else 
    return PS_ENOTINSTALLED;
#endif
}

