/* $Id: debug.c,v 1.23 2006/01/16 21:15:02 rkufrin Exp $ */

/****************************************************************************
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *        Copyright(C) 2002-2006, The Board of Trustees of the
 *            University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *           National Center for Supercomputing Applications
 *             University of Illinois at Urbana-Champaign
 * 
 *                  http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * debug.c : PerfSuite internal debugging support
 *           Enabled at compile time.
 *
 * Author: Rick Kufrin
 * Date:   2/2003
 *
 * This file is part of PerfSuite.
 *
 */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#if _REENTRANT
# include <pthread.h>
#endif

#include <stdio.h>

#if HAVE_STDLIB_H
# include <stdlib.h>
#endif

#if HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#if HAVE_UNISTD_H
# include <unistd.h>
#endif

#if DEBUG

#include <perfsuite.h>
#include "psdebug.h"

#if _REENTRANT
static pthread_once_t _ps_debug_once = PTHREAD_ONCE_INIT;
#endif

ps_debuglevel_t _ps_debuglevel = PS_DEBUG_UNINITIALIZED;

BEGIN_C_DECLS
static void ps_internal_setdebuglevel PARAMS((void));
END_C_DECLS

/* @@ Change this to use a mutex */

static void
ps_internal_setdebuglevel(void)
{
    char *envval;

    if ( _ps_debuglevel == PS_DEBUG_UNINITIALIZED ) {
        envval = getenv(PS_DEBUGENV);
        if ( envval != NULL ) {
            _ps_debuglevel = atoi(envval);
        }
        else {
            _ps_debuglevel = PS_DEBUG_OFF;
        }
        fputs("PerfSuite debugging enabled (debug level: ", stdout);
        switch ( _ps_debuglevel ) {
        case -1:
        case  0: fputs("PS_DEBUG_OFF", stdout); break;
        case  1: fputs("PS_DEBUG_FATAL", stdout); break;
        case  2: fputs("PS_DEBUG_WARNING", stdout); break;
        case  3: fputs("PS_DEBUG_INFO", stdout); break;
        case  4: fputs("PS_DEBUG_VERBOSE", stdout); break;
        default: fprintf(stdout, "other (%d)",_ps_debuglevel); break;
        }
        fputs(")", stdout);
        printf(" [PID %d]\n", getpid());
        fputs("Library version: ", stdout);
#if _REENTRANT
        puts("threaded");
#else
        puts("non-threaded");
#endif        
    }

    return;
}

ps_status_t
ps_setdebuglevel(void)
{
#if _REENTRANT
    pthread_once(&_ps_debug_once, ps_internal_setdebuglevel);
#else
    ps_internal_setdebuglevel();
#endif

    return PS_SUCCESS;
}

ps_debuglevel_t
ps_debug(void)
{
    return _ps_debuglevel;
}

#endif /* DEBUG */

void
ps_i_love_you(void)
{
    puts(":)");
}
