/* $Id: malloc.c,v 1.11 2006/01/16 21:15:02 rkufrin Exp $ */

/****************************************************************************
 *
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *          Copyright(C) 2003-2006, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>

#if HAVE_STDLIB_H
# include <stdlib.h>
#endif

#if HAVE_MALLOC_H
# include <malloc.h>
#endif

#include <perfsuite.h>
#include <psdebug.h>

#define MALLOC_FAIL "out of memory"

static void
fatal(char *msg)
{
    fprintf(stderr, "PerfSuite internal error: %s\n", msg);
    exit(1);
}

void *
_ps_malloc(size_t bytes)
{
    void *mem = malloc(bytes);

    if ( ! mem ) {
       fatal(MALLOC_FAIL);
    }

    return mem;
}

int
ps_dmemusage(float *total_mb, float *used_mb, float *free_mb)
{
#if HAVE_MALLINFO
    struct mallinfo minfo;

    minfo = mallinfo();

    PS_DEBUG(PS_DEBUG_VERBOSE, \
             printf("uordblks = %d, fordblks = %d, hblkhd = %d\n", \
             minfo.uordblks, minfo.fordblks, minfo.hblkhd);)

    *total_mb = (float) (minfo.arena + minfo.hblkhd) / 1048576.0;
    *used_mb  = (float) minfo.uordblks / 1048576.0;
    *free_mb  = (float) minfo.fordblks / 1048576.0;

    return PS_SUCCESS;
#else
    return PS_ENOTIMPLEMENTED;
#endif
}

#define PSF_DMEMUSAGE_F77 F77_FUNC_(psf_dmemusage,PSF_DMEMUSAGE)

void
PSF_DMEMUSAGE_F77(float *total_mb, float *used_mb, float *free_mb, int *ierr)
{
    *ierr = ps_dmemusage(total_mb, used_mb, free_mb);

    return;
}
