/* $Id: process.c,v 1.8 2006/01/16 21:15:02 rkufrin Exp $ */

/****************************************************************************
 *
 *                      University of Illinois/NCSA
 *                          Open Source License
 * 
 *          Copyright(C) 2004-2006, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                             Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * process.c: routines having to do with process management
 *
 * Author: Rick Kufrin
 * Date:   6/29/2004
 *
 * This file is part of PerfSuite.
 */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#if HAVE_UNISTD_H
# include <unistd.h>
#endif

#if HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#if HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif

#if HAVE_SYS_SYSCALL_H
# include <sys/syscall.h>
#endif

#if HAVE_FCNTL_H
# include <fcntl.h>
#endif

#if HAVE_ERRNO_H
# include <errno.h>
#endif

#include <perfsuite.h>
#include "psinternal.h"

/*
 * _ps_daemon: make the caller into a daemon.
 *
 * This routine is based on advice from the comp.unix.programming FAQ.
 *
 */

int
_ps_daemon(char *dir)
{
    pid_t pid, sid;
    int fd, maxfd, ret, i;

    pid = fork();
    switch ( pid ) {
    case 0:
        break;
    case -1:
        return(PS_ESYS);
    default:
        _exit(0);
    }

    sid = setsid();
    if ( sid < 0 ) {
        return(PS_ESYS);
    }

    pid = fork();
    switch ( pid ) {
    case 0:
        break;
    case -1:
        return(PS_ESYS);
    default:
        _exit(0);
    }

    if ( dir != NULL ) {
        ret = chdir(dir);
        if ( ret != 0 ) {
            return(PS_ESYS);
        }
    }

    maxfd = sysconf(_SC_OPEN_MAX);

    for ( fd = 0; fd < maxfd; fd++ ) {
        ret = close(fd);
        if ( ret == -1 ) {
            if ( errno == EBADF ) {
                continue;
            }
            return(PS_ESYS);
        }
    }

    fd = open("/dev/null", O_RDWR);
    if ( fd == -1 ) {
        return(PS_ESYS);
    }

    for ( i = 1; i < 3; i++ ) {
        ret = dup(fd);
        if ( ret == -1 ) {
            return(PS_ESYS);
        }
    }

    return PS_SUCCESS;
}

int
_ps_gettid(pid_t *tid)
{
    pid_t id;

#if HAVE_GETTID
    id = (pid_t) gettid();
#elif HAVE_SYSCALL && defined(SYS_gettid)
    id = (pid_t) syscall(SYS_gettid);
#else
    id = getpid();
#endif
    if ( id < 0 ) {
        return PS_ESYS;
    }

    *tid = id;

    return PS_SUCCESS;
}
