/* $Id: pserror.h,v 1.24 2006/01/16 21:15:02 rkufrin Exp $ */

/****************************************************************************
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *        Copyright(C) 2002-2006, The Board of Trustees of the
 *            University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *           National Center for Supercomputing Applications
 *             University of Illinois at Urbana-Champaign
 * 
 *                  http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

#ifndef PSERROR_H_INCLUDED
#define PSERROR_H_INCLUDED 1

/*
 * Maximum length of an error message returned by PerfSuite.
 */

#define PS_MAXERRLEN 64

/*
 * Error / status codes for PerfSuite.
 *
 * Each sub-package has a range of errors assigned for use.
 * Successful completion of any routine returns PS_SUCCESS,
 * errors caught elsewhere (not within PerfSuite) return
 * PS_ENOMEM, PS_ESYS or may be propagated up from the underlying
 * software.
 *
 * Add new error codes with explicit values (not enum-generated)
 * so that it's easy to match up the error code returned with
 * its symbolic name.
 */

typedef enum {
    /***********************************/
    /* All's right with the world      */
    /***********************************/
    PS_SUCCESS                  =    0,

    /***********************************/
    /* System errors                   */
    /***********************************/
    PS_ENOMEM                   =   -2,
    PS_ESYS                     =   -1, 

    /***********************************/
    /* libperfsuite  (codes    1- 999) */
    /***********************************/
    PS_ECPUNOTSUPPORTED         =    1,
    PS_EUNKNOWNCPU              =    2,
    PS_ENOTFOUND                =    3,
    PS_ENOTINSTALLED            =    4,
    PS_EBADXMLDOCTYPE           =    5,
    PS_ENOTIMPLEMENTED          =    6,
    PS_EBUFFER                  =    7,
    PS_EINVAL                   =    8,
    PS_EEXISTS                  =    9,

    /***********************************/
    /* libpshwpc     (codes 1000-1999) */
    /***********************************/
    PS_HWPC_EXMLPARSE           = 1001,
    PS_HWPC_EEVENTSEXCEEDED     = 1002,
    PS_HWPC_EEVENTNOTSUPPORTED  = 1003,
    PS_HWPC_EBADCLASS           = 1004,
    PS_HWPC_ESTREAM             = 1005,
    PS_HWPC_EPERF               = 1006,
    PS_HWPC_EBADEVENT           = 1007,
    PS_HWPC_ENOSUCHEVENT        = 1008,
    PS_HWPC_EEVENTNOTAVAILABLE  = 1009,
    PS_HWPC_ENOTSUPPORTED       = 1010,
    PS_HWPC_ENOSUCHXMLDOC       = 1011,
    PS_HWPC_ESEQUENCE           = 1012,
    PS_HWPC_EMODE               = 1013

    /***********************************/
    /* libpspmpi     (codes 2000-2999) */
    /***********************************/

} ps_status_t;

/*
 * Minimum and maximum error codes currently defined.
 */

#define PS_MINERRNO PS_ENOMEM
#define PS_MAXERRNO PS_HWPC_EMODE

#endif /* PSERROR_H_INCLUDED */
