/* $Id: pstypes.h,v 1.36 2009/11/25 20:12:11 rkufrin Exp $ */

/****************************************************************************
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *        Copyright(C) 2002-2009, The Board of Trustees of the
 *            University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *           National Center for Supercomputing Applications
 *             University of Illinois at Urbana-Champaign
 * 
 *                  http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

#ifndef PSTYPES_H_INCLUDED
#define PSTYPES_H_INCLUDED 1

#include <time.h>
#include <sys/types.h>
#include <unistd.h>

/*
 * procfs definitions (procfs.c)
 */

/*
 * Structure definition for accessing "stat" files in the 
 * /proc filesystem.  This applies to Linux 2.4
 *
 * @@ These are wrong.   Check the task_struct definition
 * in /usr/src/linux/include/linux/sched.h
 */

#define MAXCOMMSIZ 16

typedef struct {
    pid_t pid;               /* Process ID */

    char comm[MAXCOMMSIZ];   /* The filename of the executable, in
                              * parentheses.  This is visible     
                              * whether or not the executable is  
                              * swapped out.
                              */

    char state;              /* One character from the string
                              * "RSDZT" where R is running, S is
                              * sleeping in an interruptible wait, D
                              * is sleeping in an uninterruptible
                              * wait or swapping, Z is zombie, and T
                              * is traced or stopped (on a signal).
                              */

    pid_t ppid;              /* The PID of the parent. */

    pid_t pgrp;              /* The process group ID of the process. */

    pid_t session;           /* The session ID of the process. */

    int tty;                 /* The tty the process uses. */

    int tpgid;               /* The process group ID of the process
                              * which currently owns the tty that
                              * the process is connected to.
                              */

    unsigned long flags;     /* The flags of the process.
                              * Currently, every flag has the math
                              * bit set, because crt0.s checks for
                              * math emulation, so this is not
                              * included in the output.  This is
                              * probably a bug, as not every process
                              * is a compiled C program.  The math
                              * bit should be a decimal 4, and the
                              * traced bit is a decimal 10.
                              */

    unsigned long minflt;    /* The number of minor faults the
                              * process has made, those which have
                              * not required loading a memory page
                              * from disk.
                              */

    unsigned long cminflt;   /* The number of minor faults that the
                              * process and its children have made.
                              */

    unsigned long majflt;    /* The number of major faults the
                              * process has made, those which have
                              * required loading a memory page from
                              * disk.
                              */

    unsigned long cmajflt;   /* The number of major faults that the
                              * process and its children have made.
                              */

    clock_t utime;           /* The number of jiffies that this
                              * process has been scheduled in user
                              * mode.
                              */

    clock_t stime;           /* The number of jiffies that this
                              * process has been scheduled in kernel
                              * mode.
                              */

    clock_t cutime;          /* The number of jiffies that this
                              * process and its children have been
                              * scheduled in kernel mode.
                              */

    clock_t cstime;          /* The number of jiffies that this
                              * process and its children have been
                              * scheduled in kernel mode.
                              */

    long priority;           /* The standard nice value, plus
                              * fifteen.  The value is never
                              * negative in the kernel.
                              */

    long nice;               /*
                              *
                              */

    long numthreads;         /* 2.6
                              *
                              */

    unsigned long itrealvalue; /* The time (in jiffies) before the
                              * next SIGALRM is sent to the process
                              * due to an interval timer.
                              */

    unsigned long starttime; /* Time the process started in jiffies
                              * after system boot.
                              */

    unsigned long vsize;     /* Virtual memory size */

    unsigned long rss;       /* Resident Set Size: number of pages
                              * the process has in real memory,
                              * minus 3 for administrative purposes.
                              * This is just the pages which count
                              * towards text, data, or stack space.
                              * This does not include pages which
                              * have not been demand-loaded in, or
                              * which are swapped out.
                              */

    unsigned long rlim;      /* Current limit in bytes on the rss of
                              * the process (usually 2,147,483,647).
                              */

    unsigned long startcode; /* The address above which program text
                              * can run.
                              */

    unsigned long endcode;   /* The address below which program text
                              * can run.
                              */

    unsigned long startstack;/* The address of the start of the
                              * stack.
                              */

    unsigned long kstkesp;   /* The current value of esp (32-bit
                              * stack pointer), as found in the
                              * kernel stack page for the process.
                              */

    unsigned long kstkeip;   /* The current EIP (32-bit instruction
                              * pointer).
                              */

    int signal;              /* The bitmap of pending signals
                              * (usually 0).
                              */

    int blocked;             /* The bitmap of blocked signals
                              * (usually 0, 2 for shells).
                              */

    int sigignore;           /* The bitmap of ignored signals. */

    int sigcatch;            /* The bitmap of catched signals. */

    unsigned long wchan;     /* This is the "channel" in which the
                              * process is waiting.  This is the
                              * address of a system call, and can be
                              * looked up in a namelist if you need
                              * a textual name. (If you have an up-
                              * to-date /etc/psdatabase, then try ps -l
                              * to see the WCHAN field in action)
                              */

    unsigned long nswap;     /*
                              *
                              */

    unsigned long cnswap;    /*
                              *
                              */

    int exitsignal;          /*
                              *
                              */

    int processor;           /*
                              *
                              */
} ps_procstat_t;

/* #defines to help keep the Fortran interface straight - these
   are all one less than their equivalents in fperfsuite.h
 */

#define PS_PID          0
#define PS_PPID         1
#define PS_PGRP         2
#define PS_SESSION      3
#define PS_TTY          4
#define PS_TPGID        5
#define PS_FLAGS        6
#define PS_MINFLT       7
#define PS_CMINFLT      8
#define PS_MAJFLT       9
#define PS_CMAJFLT     10
#define PS_UTIME       11
#define PS_STIME       12
#define PS_CUTIME      13
#define PS_CSTIME      14
#define PS_PRIORITY    15
#define PS_NICE        16
#define PS_NUMTHREADS  17
#define PS_ITREALVALUE 18
#define PS_STARTTIME   19
#define PS_VSIZE       20
#define PS_RSS         21
#define PS_RLIM        22
#define PS_STARTCODE   23
#define PS_ENDCODE     24
#define PS_STARTSTACK  25
#define PS_KSTKESP     26
#define PS_KSTKEIP     27
#define PS_SIGNAL      28
#define PS_BLOCKED     29
#define PS_SIGIGNORE   30
#define PS_SIGCATCH    31
#define PS_WCHAN       32
#define PS_NSWAP       33
#define PS_CNSWAP      34
#define PS_EXITSIGNAL  35
#define PS_PROCESSOR   36


/*
 * timers definitions (timers.c)
 */

typedef struct {
    unsigned long secs;
    unsigned long msecs;
} ps_time_t;

#endif /* PSTYPES_H_INCLUDED */
