/* $Id: signals.h,v 1.8 2011/03/16 20:53:40 ruiliu Exp $ */

/****************************************************************************
 *
 *                      University of Illinois/NCSA
 *                          Open Source License
 * 
 *     Copyright(C) 2004-2006, 2010-2011, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                             Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

#ifndef SIGNALS_H_INCLUDED
#define SIGNALS_H_INCLUDED 1

#if HAVE_CONFIG_H
# include <config.h>
#endif

#if HAVE_SIGNAL_H
# include <signal.h>
#endif

#if HAVE_UCONTEXT_H
# include <ucontext.h>
#elif HAVE_SYS_UCONTEXT_H
# include <sys/ucontext.h>
#endif

#define PS_DEFAULT_PROFILING_SIGNAL SIGUSR1
#define PS_PROFILING_FLAGS          SA_SIGINFO

#if (PPC32 || PPC64)
#  define USE_MCONTEXT 1
#  define USE_SIGCONTEXT 0
#else  /* All other architectures. */
#  define USE_MCONTEXT 0
#  define USE_SIGCONTEXT 1
#endif

static inline caddr_t
get_pc(void *p)
{
    struct ucontext *uc = (struct ucontext *) p;
    caddr_t pc;
#if USE_SIGCONTEXT
    struct sigcontext *sc;
    sc = (struct sigcontext *) &uc->uc_mcontext;
# if X86_64
    pc = (caddr_t) sc->rip;
# elif IA32
    pc = (caddr_t) sc->eip;
# elif IA64
    pc = (caddr_t) sc->sc_ip;
# else
#  error "profile handler not defined for this architecture"
# endif
#elif USE_MCONTEXT  /* Defined to 1 only for PPC64 and PPC32. */
    mcontext_t *mc;
# if (SIZEOF_VOID_P == 8)  /* PPC64. */
    mc = &uc->uc_mcontext;
    pc = (caddr_t) mc->regs->nip;
# else /* PPC32. */
    mc = uc->uc_mcontext.uc_regs;
#   if  HAVE_ASM_PTRACE_H
    pc = (caddr_t) mc->gregs[PT_NIP];
#   else  /* Does not have asm/ptrace.h, use the hardcoded value of 32. */
#define REG_NIP 32
    pc = (caddr_t) mc->gregs[REG_NIP];
#   endif
# endif
#else
# error "No context usage defined"
#endif /* USE_SIGCONTEXT */

    return pc;
}

#endif /* SIGNALS_H_INCLUDED */
