/* $Id: eventlist.h,v 1.23 2007/09/03 14:18:55 rkufrin Exp $ */

/****************************************************************************
 *
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *          Copyright(C) 2002-2006, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

#ifndef _PS_EVENTLIST_INCLUDED
#define _PS_EVENTLIST_INCLUDED

#include <perfsuite.h>
/*#include "hwpc.h"*/

/*
 * This corresponds to the maximum number of events that
 * PAPI can multiplex.  We'll use that as a chunksize for
 * allocating blocks of memory for event data in the XML
 * document.
 */

#define MAXEVENTS 32

/* The ID of the supporting counter access software */
typedef enum {
  UNKNOWN=-1,
  HWPCNULL,
  ITIMER,
  PAPI,
  PERFMON
} ps_hwpc_class_t;

/* The types of  events accepted */
typedef enum {
  NATIVE,
  PRESET
} ps_hwpc_eventtype_t;

/* The counting domains accepted */
typedef enum {
  DOMAINUSER,
  DOMAINKERNEL,
  DOMAINALL
} ps_hwpc_domaintype_t;

/* A description of an individual event */
/* @@ To add a descriptive piece of text for a PAPI native
 * event, we will probably want to add a character pointer
 * here.  PAPI lets you look it up at runtime, so it wouldn't
 * be necessary in that case (although we could retrieve it
 * early and store it if we wanted to).  If we really
 * want to be space misers, we can have a union shared between
 * "derived" (which only makes sense for PAPI) and this
 * descriptive text.
 */
typedef struct ps_event {
  ps_hwpc_eventtype_t type;
  int                 derived; /* Unknown till runtime */
  char                *val;
} ps_hwpc_event_t;

/* A vector of events, including their class */
typedef struct ps_eventArray {
  ps_hwpc_class_t class;
  ps_hwpc_event_t *events;
  int             numEvents;
} ps_hwpc_eventarray_t;

/* XML tags pertinent to events */
#define EVENTLISTTAG       "ps_hwpc_eventlist"

#define EVENTLISTCLASSATT  "class"
#define ITIMERCLASSATTVAL  "itimer"
#define NULLCLASSATTVAL    "null"
#define PAPICLASSATTVAL    "PAPI"
#define PERFMONCLASSATTVAL "perfmon"

#define EVENTLISTDOMAINATT "domain"
#define USERDOMAINATTVAL   "user"
#define KERNELDOMAINATTVAL "kernel"
#define ALLDOMAINATTVAL    "all"

#define EVENTTAG           "ps_hwpc_event"
#define EVENTTAGTYPEATT    "type"
#define EVENTTAGNAMEATT    "name"

#define NATIVEATTVAL       "native"
#define PRESETATTVAL       "preset"

/* XML tags pertinent to profiles */
#define PROFILETAG         "ps_hwpc_profile"
#define THRESHOLDATT       "threshold"

BEGIN_C_DECLS

#if 0
int _ps_parseEventListFromFile PARAMS((char *filename,
                                       ps_hwpc_globals_t* void *globals));
#endif
END_C_DECLS

/* Not used */
#define _ps_validateEventListFromFile(file) \
	_ps_parseEventListFromFile(file, (ps_hwpc_eventarray_t *)NULL, \
                                   (int *)NULL)

#endif /* _PS_EVENTLIST_INCLUDED */
