/* $Id: exclude.c,v 1.10 2006/01/21 22:36:15 rkufrin Exp $ */

/****************************************************************************
 *
 *                      University of Illinois/NCSA
 *                          Open Source License
 * 
 *          Copyright(C) 2004-2006, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                             Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * hwpc-exclude:  utility routines to support excluding specific
 *                programs from monitoring.
 *
 * Author: Rick Kufrin
 * Date:   9/24/2004
 *
 * This file is part of PerfSuite.
 */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>

#if HAVE_STRING_H
# include <string.h>
#endif

#if HAVE_CTYPE_H
# include <ctype.h>
#endif

#include <psinternal.h>
#include <psdebug.h>

#include "exclude.h"

/*
 * Utility routine that reads a flat-file database that should contain
 * full path names of executables that should be excluded from monitoring
 * with this library.
 * 
 * Returns 1 if exe is found in the database, 0 otherwise.
 */

int
_ps_hwpc_exclude_exe(char *exe, char *db)
{
    int exclude = 0;
    char buf[MAXEXCLUDEEXENAME + 1], *ptr, *index;
    FILE *fp;
#if DEBUG
    int lineno = 0;
#endif

# if _REENTRANT
    pthread_cleanup_push((void *)_ps_pthread_cleanup_fd, &fp);
# endif

    fp = fopen(db, "r");
    if ( fp != NULL ) {
        PS_DEBUG(PS_DEBUG_INFO, \
                 printf("Checking exclusion database \"%s\" for \"%s\"\n", \
                        db, exe);)
        while ( fgets(buf, MAXEXCLUDEEXENAME, fp) != NULL ) {
#if DEBUG
            lineno++;
#endif
            ptr = buf;

            /* Get rid of the newline in the buffer */
            index = strchr(ptr, '\n');
            if ( index != NULL ) {
                *index = '\0';
            }

            /* Ignore everything from the first '#' character on */
            index = strchr(ptr, '#');
            if ( index != NULL ) {
                *index = '\0';
            }

            /* Advance to the first non-whitespace character */
            while ( isspace(*ptr) ) {
                ++ptr;
            }

            /* Remove all trailing spaces */
            index = ptr + strlen(ptr) - 1;
            while ( isspace(*index) ) {
                --index;
            }
            *(index+1) = '\0';

            if ( strcmp(ptr, exe) == 0 ) {
                exclude = 1;
                break;
            }
        }
        fclose(fp);
    }
    else {
        PS_DEBUG(PS_DEBUG_WARNING, \
                 printf("could not open exclusion database \"%s\".\n", db);)
    }

#if _REENTRANT
    pthread_cleanup_pop(0);
#endif

    PS_DEBUG(PS_DEBUG_INFO, \
             if ( exclude ) { \
                 printf("program \"%s\" excluded from measurement.\n", exe); \
                 printf("found on line %d in database \"%s\".\n", lineno, db); \
             } \
             else { \
                 printf("program \"%s\" not found in database.\n", exe); \
             })

    return exclude;
}

