/* $Id: null.c,v 1.23 2013/05/07 21:51:49 ruiliu Exp $ */

/****************************************************************************
 *
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *       Copyright(C) 2003-2006, 2013, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * null: "do-nothing" layer for libpshwpc.  This is used
 *        when the user hasn't specified any events to
 *        count or has specified a class of "null".  
 *
 *        One reason why this might be useful is
 *        to obtain the generic information about a run,
 *        such as machine name, hardware config, date,
 *        wall-clock time.
 *
 *        The other reason this layer might be used is to use
 *        a time-based (vs counter-based) method for profiling.
 *
 * Author: Rick Kufrin
 * Date:   2/2003
 *
 * This file is part of PerfSuite.
 */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#if HAVE_STDLIB_H
# include <stdlib.h>
#endif

#if HAVE_GNU_LIBC_VERSION_H
# include <gnu/libc-version.h>
#endif

#include <perfsuite.h>
#include "hwpc.h"
#include "null.h"

int
get_num_counters_null()
{
    return 0;
}

int
get_max_counters_null(ps_hwpc_globals_t *globals)
{
    return 0;
}

int
init_counters_null(ps_hwpc_globals_t *globals)
{
#if HAVE_GNU_LIBC_VERSION_H
    globals->version = (char *) gnu_get_libc_version();
#endif

    return PS_SUCCESS;
}

int
start_counters_null(ps_hwpc_globals_t *globals)
{
    return PS_SUCCESS;
}

int
read_counters_null(ps_hwpc_value_t *values)
{
    return PS_SUCCESS;
}

int
stop_counters_null(ps_hwpc_value_t *values)
{
    return PS_SUCCESS;
}

int
destroy_counters_null()
{
    return PS_SUCCESS;
}

int
init_profiling_null(ps_hwpc_globals_t *globals, ps_hwpc_sample_t **samples)
{
    return PS_SUCCESS;
}

int
start_profiling_null(ps_hwpc_globals_t *globals, ps_hwpc_sample_t **samples)
{
#if HAVE_PROFIL
    profil(samples[0], globals->profiles.profmap[0].nbins,
           (size_t) globals->profiles.profmap[0].offset,
           globals->profiles.profmap[0].scale);
#endif

    return PS_SUCCESS;
}

/*
 * @@ There seem to be conflicting definitions of how to turn off
 * profiling, either by passing a buffer value of NULL or specifying
 * a scale factor (4th argument) of 0.  Should be resolved, this
 * was changed in 0.6.2a3 to address compiler complaints.
 *
 * The NULL buffer convention was described in man pages marked Linux 2.0
 */
int
suspend_profiling_null(ps_hwpc_globals_t *globals, ps_hwpc_sample_t **samples)
{
#if HAVE_PROFIL
    profil(samples[0], 0, 0, 0);
#endif

    return PS_SUCCESS;
}

int
stop_profiling_null(ps_hwpc_globals_t *globals, ps_hwpc_sample_t **samples)
{
#if HAVE_PROFIL
    profil(samples[0], 0, 0, 0);
#endif

    return PS_SUCCESS;
}

