/* $Id: papicompat.c,v 1.20 2006/01/21 22:36:15 rkufrin Exp $ */

/****************************************************************************
 *
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *          Copyright(C) 2003-2006, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * papicompat.c: PAPI 2 compatibility functions for hwpc-papi.c
 *
 * Author: Rick Kufrin
 * Date:   11/2003
 *
 * This file is derived from code in the PAPI library from the
 * Innovative Computing Laboratory at the University of Tennessee.
 *
 * The PAPI license follows:

-----------------------------------------------------------------------------

                         Copyright 1997-2001 PAPI Team,
                           Innovative Computing Labs
                          Computer Science Department,
                            University of Tennessee,
                                 Knoxville, TN.
                              All Rights Reserved.
                                                                                
ALLOWED USAGE:
                                                                                
Users may use PAPI in any capacity they wish. We only ask that proper credit
and citations be used when the PAPI system is being leveraged in other
software systems.
                                                                                
REDISTRIBUTION:
                                                                                
Users are allowed to freely distribute the PAPI system in unmodified form. At
no time is a user to accept monetary or other compensation for redistributing
parts or all of the PAPI system.
                                                                                
MODIFICATION OF CODE:
                                                                                
Users are free to make whatever changes they wish to the PAPI system to suit
their individual needs. We mandate, however, that you clearly highlight which
portions are of the original system and which are a result of the third-party
modification.
                                                                                
WARRANTY DISCLAIMER:
                                                                                
USER ACKNOWLEDGES AND AGREES THAT:
        (A) NEITHER THE PAPI TEAM NOR THE BOARD OF REGENTS OF THE UNIVERSITY OF
            TENNESSEE SYSTEM (REGENTS) MAKE ANY REPRESENTATIONS OR WARRANTIES
            WHATSOEVER ABOUT THE SUITABILITY OF PAPI FOR ANY PURPOSE;
        (B) PAPI IS PROVIDED ON AN "AS IS, WITH ALL DEFECTS" BASIS WITHOUT
            EXPRESS OR IMPLIED WARRANTIES, INCLUDING WARRANTIES OF MERCHANT-
            ABILITY AND FITNESS FOR A PARTICULAR PURPOSE OR NONINFRINGEMENT;
        (C) NEITHER THE PAPI TEAM NOR THE REGENTS SHALL BE LIABLE FOR ANY
            DAMAGE OR LOSS OF ANY KIND ARISING OUT OF OR RESULTING FROM USER'S
            POSSESSION OR USE OF PAPI (INCLUDING DATA LOSS OR CORRUPTION),
            REGARDLESS OF WHETHER SUCH LIABILITY IS BASED IN TORT, CONTRACT,
            OR OTHERWISE; AND
        (D) NEITHER THE PAPI TEAM NOR THE REGENTS HAVE AN OBLIGATION TO PROVIDE
            DEBUGGING, MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
            MODIFICATIONS EXCEPT WHERE EXPLICIT WRITTEN ARRANGEMENTS HAVE BEEN
            PRE-ARRANGED.
DAMAGES DISCLAIMER:
                                                                                
USER ACKNOWLEDGES AND AGREES THAT IN NO EVENT WILL THE PAPI TEAM OR THE REGENTS
BE LIABLE TO USER FOR ANY SPECIAL, CONSEQUENTIAL, INDIRECT OR SIMILAR DAMAGES,
INCLUDING ANY LOST PROFITS OR LOST DATA ARISING OUT OF THE USE OR INABILITY TO
USE PAPI EVEN IF THE PAPI TEAM OR THE REGENTS HAVE BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.
                                                                                
ATTRIBUTION REQUIREMENT:
                                                                                
User agrees that any reports, publications, or other disclosure of results
obtained with PAPI will attribute its use by an appropriate citation. The
appropriate reference for PAPI is "The PAPI Library (PAPI) was developed by
the PAPI Team at the Computer Science Department of the University of
Tennessee, Knoxville. All rights, title, and interest in PAPI are owned by
the PAPI Team."
                                                                                
COMPLIANCE WITH APPLICABLE LAWS:
                                                                                
User agrees to abide by copyright law and all other applicable laws of the
United States including, but not limited to, export control laws.
-----------------------------------------------------------------------------
 */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#if PAPI3

# include <stdio.h>

# if HAVE_STDLIB_H
#  include <stdlib.h>
# endif

# if HAVE_STRING_H
#  include <string.h>
# endif

# if ! HAVE_STRDUP
#  error "strdup() is not available on this system"
# endif

# if HAVE_ASSERT_H
#  include <assert.h>
# else
#  ifndef NDEBUG
#   define assert(x) \
      if ( ! (x) ) { \
           fprintf(stderr, "Assertion failed in file %s, line %d\n", \
                           __FILE__, __LINE__); \
           abort(); \
      }
#  else
#   define assert(x)
# endif
#endif

#include <papicompat.h>

static int
copy_fields(PAPI_event_info_t *info3, PAPI_preset_info_t *info2)
{
    if (((info2->event_name  = strdup(info3->symbol))      == NULL) ||
        ((info2->event_descr = strdup(info3->long_descr))  == NULL) ||
        ((info2->event_label = strdup(info3->short_descr)) == NULL) ||
#if HAVE_PAPI_VENDOR_NAME
        /* Structure fields changed during PAPI 3 beta */
        ((info2->event_note  = strdup(info3->vendor_name)) == NULL)
#else
        ((info2->event_note  = strdup(info3->note)) == NULL)
#endif
       ) {
        return PAPI_ENOMEM;
    }
    info2->event_code = info3->event_code;
    info2->avail      = (info3->count != 0);
    if ( info3->count > 1 ) {
        info2->flags |= PAPI_DERIVED;
    } else {
        info2->flags &= ~PAPI_DERIVED;
    }
    return PAPI_OK;
}

int
PAPI_query_event_verbose(int code, PAPI_preset_info_t *info)
{
    PAPI_event_info_t info3;
    int papiret, ret;
    
    papiret = PAPI_get_event_info(code, &info3);
    if ( papiret == PAPI_OK ) {
        if ( info3.count == 0 ) {
            ret = PAPI_ENOEVNT;
        }
        else {
            ret = copy_fields(&info3, info);
        }
    }
    else {
        ret = papiret;
    }

    return ret;
}

const PAPI_preset_info_t *
PAPI_query_all_events_verbose(void)
{
    PAPI_event_info_t info3;
    PAPI_preset_info_t *info2;
    int i = PAPI_PRESET_MASK, avail_only = 0, event = 0;

    info2 = (PAPI_preset_info_t *)
          malloc(PAPI_MAX_PRESET_EVENTS * sizeof(PAPI_preset_info_t));
    if ( info2 != NULL ) {
        do {
            if ( PAPI_get_event_info(i, &info3) == PAPI_OK ) {
                if ( copy_fields(&info3, &info2[event]) != PAPI_OK ) {
                    return NULL;
                }
                event++;
            }
        } while ( PAPI_enum_event(&i, avail_only) == PAPI_OK );
        assert( event < PAPI_MAX_PRESET_EVENTS );
        info2[event].event_name = NULL;
    }

    return info2;
}
#endif /* PAPI3 */
