/* $Id: pshwpc.h,v 1.25 2006/01/21 22:36:15 rkufrin Exp $ */

/****************************************************************************
 *
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *          Copyright(C) 2002-2006, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

#ifndef PS_HWPC_INCLUDED
#define PS_HWPC_INCLUDED 1

#include <perfsuite.h>

/*
 * Base size of a performance counter value.
 */

typedef long long ps_hwpc_value_t;

BEGIN_C_DECLS

/*
 * Only a single thread should call these,
 * and only once.
 */
extern int ps_hwpc_init     PARAMS((void));
extern int ps_hwpc_shutdown PARAMS((void));

/*
 * All threads can call these any number of
 * times (must have been preceded by a single
 * call to ps_hwpc_init()).
 *
 * ps_hwpc_read() cannot be used in profiling mode.
 */
extern int ps_hwpc_start   PARAMS((void));
extern int ps_hwpc_suspend PARAMS((void));
extern int ps_hwpc_read    PARAMS((ps_hwpc_value_t *values)); 

/*
 * All threads can (should) call this,
 * once only.
 */
extern int ps_hwpc_stop PARAMS((const char *fname));

/*
 * Single call to enable psrun-like behavior for
 * whole-program monitoring from the point of call.
 *
 * For use with serial (not POSIX threads) programs.
 */
extern int ps_hwpc_psrun PARAMS((void));

/*
 * Inquiry routines.
 */
extern int ps_hwpc_eventnames PARAMS((char ***eventnames));
extern int ps_hwpc_numevents  PARAMS((int *numevents));
extern int ps_hwpc_state      PARAMS((int *state));
extern int ps_hwpc_version    PARAMS((int *major, int *minor, int *release,
                                      int *type));

/*
 * Routine to provide output of PerfSuite XML documents with
 * user-collected data.  Cannot be used in profiling mode.
 */
#if 0
extern int ps_hwpc_xml_write PARAMS((const char *filename,
                                     const char *annotation,
                                     ps_hwpc_value_t *values, long long ticks,
                                     double *usersecs, double *systemsecs));
#endif

END_C_DECLS

#endif /* PS_HWPC_INCLUDED */
