/* $Id: demangle.c,v 1.5 2011/09/15 21:55:56 ruiliu Exp $ */

/****************************************************************************
 *
 *                      University of Illinois/NCSA
 *                          Open Source License
 * 
 *          Copyright(C) 2010-2011, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                             Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * Author: Rick Kufrin, Rui Liu
 * Date:   May 2010
 *
 * This file is part of PerfSuite.
 */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <perfsuite.h>
#include <psx.h>


#if HAVE_DEMANGLE_H
# include <demangle.h>
#else
/* Declare the cplus_demangle function to be used later. */
extern char * cplus_demangle (const char *mangled, int options);
#endif

#ifndef DMGL_PARAMS
#define DMGL_PARAMS      (1 << 0)       /* Include function args */
#define DMGL_ANSI        (1 << 1)       /* Include const, volatile, etc */
#endif

#if CPLUS_DEMANGLE_PIC

/*
  _psx_demangle: Perform function name demangling. A thin wrapper
                 around cplus_demangle() in libiberty or libbfd.

  int
  _psx_demangle(const char *inStr, const char *style, char **outStr);

  Arguments:
      inStr:  a possibly mangled function/method name;
      style:  a demangling style string. The valid values are:
              "none", "auto", "gnu", "lucid", "arm", "hp", "edg",
              "gnu-v3", "java", "gnat".
      outStr: a pointer to a character pointer.
              If demangling succeeds, a string is allocated to hold
              the demangled function name. Upon return, "outStr" will
              be set to point to it. The caller is responsible to free
              the memory.

  Return value:
      A status code indicating success (PS_SUCCESS) or failure
      (error codes that are different from PS_SUCCESS).

  Error conditions:
      1) PS_EINVAL: if inStr or style is NULL, or starts with '\0',
         or if the style given is unsupported, that is, it is not one
         of the values listed above in the "Arguments" section;
      2) PS_ENOTINSTALLED: if cplus_demangle() is not available.
*/

int
_psx_demangle(const char *inStr, const char *inStyle, char **outStr)
{
    /* enum demangling_styles style; */
    char * alloc = NULL;

    if ( (NULL == inStr) || (0 == *inStr) ||
         (NULL == inStyle) || (0 == *inStyle) ) {
        return PS_EINVAL; /* invalid argument */
    }

/*
#if HAVE_CPLUS_DEMANGLE_NAME_TO_STYLE
    style = cplus_demangle_name_to_style (inStyle);
    if (unknown_demangling == style) {
        return PS_EINVAL;
    }
    cplus_demangle_set_style (style);
#endif
*/

    alloc = cplus_demangle (inStr, DMGL_PARAMS | DMGL_ANSI);
    *outStr = alloc;
    return PS_SUCCESS;
}

#else /* ! CPLUS_DEMANGLE_PIC */

int
_psx_demangle(const char *inStr, const char *style, char **outStr)
{
    return PS_ENOTINSTALLED;
}
#endif
