/* $Id: Papi_timers.c,v 1.8 2006/01/16 21:21:37 rkufrin Exp $ */

/****************************************************************************
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *          Copyright(C) 2003-2006, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications 
 *              University of Illinois at Urbana-Champaign
 * 
 *                  http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

#include "Papi.h"

/*
 * Papi_timers.c - Tcl interface to PAPI's high-resolution timers
 *
 * Syntax: ::papi::time subcommand ?option?
 *
 * Subcommand is either "process" or "real" and is required
 * Option is either "-cycles" or "-usecs" (default is -usecs)
 *
 * Author: Rick Kufrin
 * Date:   5/3/2003
 *
 * This file is part of PerfSuite.
 */

typedef enum {
    USECS,
    CYCLES
} papi_timeunit_t;

#define BADOPT "invalid option: must be \"-usecs\" (default) or \"-cycles\""
#define BADCMD "invalid subcommand: must be \"process\" or \"real\""

int
PAPI_timeObjCmd(ClientData clientData, Tcl_Interp *interp,
                 int objc, Tcl_Obj *CONST objv[])
{
    char *cmd, *arg=NULL, result[32];
    long long papitime = -1;
    papi_timeunit_t type = USECS;
    Tcl_Obj *res;

    if ( (objc < 2) || (objc > 3) ) {
        Tcl_WrongNumArgs(interp, 1, objv,
                         "[process|real] ?[-usecs|-cycles]?");
        return TCL_ERROR;
    }

    cmd = Tcl_GetString(objv[1]);

    if ( objc == 3 ) {
        arg = Tcl_GetString(objv[2]);
        if ( strcmp(arg, "-cycles") == 0 ) {
            type = CYCLES;
        }
        else if ( strcmp(arg, "-usecs") == 0 ) {
            type = USECS;
        }
        else {
            Tcl_SetObjResult(interp, Tcl_NewStringObj(BADOPT, strlen(BADOPT)));
            return TCL_ERROR;
        }
    }

    if ( strcmp(cmd, "real") == 0 ) {
        switch (type) {
        case USECS:
            papitime = PAPI_get_real_usec();
            break;
        case CYCLES:
            papitime = PAPI_get_real_cyc();
            break;
        default:
            break;
        }
    }
    else if ( strcmp(cmd, "process") == 0 ) {
        switch (type) {
        case USECS:
            papitime = PAPI_get_virt_usec();
            break;
        case CYCLES:
            papitime = PAPI_get_virt_cyc();
            break;
        default:
            break;
        }
    }
    else {
        Tcl_SetObjResult(interp, Tcl_NewStringObj(BADCMD, strlen(BADCMD)));
        return TCL_ERROR;
    }

    sprintf(result, "%llu", papitime);

    res = Tcl_NewStringObj(result, strlen(result));

    Tcl_SetObjResult(interp, res);

    return TCL_OK;
}
