/* $Id: Bfd.h,v 1.17 2009/11/05 16:29:44 rkufrin Exp $ */

/****************************************************************************
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *         Copyright(C) 2003-2006, The Board of Trustees of the
 *             University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications 
 *              University of Illinois at Urbana-Champaign
 * 
 *                  http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

#ifndef _PSBFD_H_INCLUDED
#define _PSBFD_H_INCLUDED 1

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>

#include <tcl.h>
#include <string.h>

#include <perfsuite.h>

#if BFD_ENABLED
#include <bfd.h>
typedef struct {
    char         *exe;
    bfd          *abfd;
    asymbol      **symtab;
    unsigned int symsize;
    long         numsyms;
    bfd_vma      addr;
    char         *file;
    char         *function;
    unsigned int line;
    int          found;
} bfdinfo_t;
#endif

#define PSBFD_PACKAGE_NAME       "psbfd"
#define PSBFD_VERSION_VARNAME    "::psbfd::version"
#define PSBFD_PATCHLEVEL_VARNAME "::psbfd::patchLevel"
#define PSBFD_PACKAGE_VERSION    "0.3"
#define PSBFD_PATCHLEVEL         "0"
#define TCL_REQUIRED_VERSION     "8.3"

BEGIN_C_DECLS

int BFD_closeObjCmd   PARAMS((ClientData clientData, Tcl_Interp *interp,
                              int objc, Tcl_Obj *CONST objv[]));
int BFD_inquireObjCmd PARAMS((ClientData clientData, Tcl_Interp *interp,
                              int objc, Tcl_Obj *CONST objv[]));
int BFD_lookupObjCmd  PARAMS((ClientData clientData, Tcl_Interp *interp,
                              int objc, Tcl_Obj *CONST objv[]));
int BFD_openObjCmd    PARAMS((ClientData clientData, Tcl_Interp *interp,
                              int objc, Tcl_Obj *CONST objv[]));

END_C_DECLS

#define UNKNOWN_SYMBOL  "??"

#define PSBFD_EACTIVE       "an object file is already opened"
#define PSBFD_ENOBFD        "no object file has been opened"
#define PSBFD_ENOTFOUND     "could not map address"
#define PSBFD_ENOTINSTALLED "psbfd is not active in this installation"

#endif /* _PSBFD_H_INCLUDED */
