/* $Id: Bfd_inquire.c,v 1.8 2009/07/01 16:16:43 rkufrin Exp $ */

/****************************************************************************
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *         Copyright(C) 2003-2006, The Board of Trustees of the
 *             University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications 
 *              University of Illinois at Urbana-Champaign
 * 
 *                  http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * Bfd_inquiry.c: Tcl interface to BFD
 *
 * Author: Rick Kufrin
 * Date:   7/27/2003
 *
 * This file is part of PerfSuite.
 *
 */

#include "Bfd.h"

#define SUBCMDS \
    "subcmd must be one of: \"executable\", \"dynamic\", \"filename\", \"has_reloc\", \"has_lineno\", \"has_debug\", \"has_syms\""

int
BFD_inquireObjCmd(ClientData clientData, Tcl_Interp *interp,
                int objc, Tcl_Obj *CONST objv[])
{
#if BFD_ENABLED

    bfdinfo_t *bfdinfo = (bfdinfo_t *) clientData;
    char *subcmd;
    int result;
    flagword flags;

    /* Check to make sure a BFD is active/opened */
    if ( bfdinfo->abfd == NULL ) {
        Tcl_SetObjResult(interp, Tcl_NewStringObj(PSBFD_ENOBFD,
                                                  strlen(PSBFD_ENOBFD)));
        return TCL_ERROR;
    }

    if ( objc != 2 ) {
        Tcl_WrongNumArgs(interp, 1, objv, SUBCMDS);
        return TCL_ERROR;
    }

    subcmd = Tcl_GetString(objv[1]);

    if ( strcmp(subcmd, "filename") == 0 ) {
        Tcl_SetObjResult(interp, Tcl_NewStringObj(bfdinfo->exe,
                                                  strlen(bfdinfo->exe)));
        return TCL_OK;
    }

    flags = bfd_get_file_flags(bfdinfo->abfd);

    if ( strcmp(subcmd, "executable") == 0 ) {
        result = flags & EXEC_P;
    }
    else if ( strcmp(subcmd, "dynamic") == 0 ) {
        result = flags & DYNAMIC;
    }
    else if ( strcmp(subcmd, "has_reloc") == 0 ) {
        result = flags & HAS_RELOC;
    }
    else if ( strcmp(subcmd, "has_lineno") == 0 ) {
        result = flags & HAS_LINENO;
    }
    else if ( strcmp(subcmd, "has_debug") == 0 ) {
        result = flags & HAS_DEBUG;
    }
    else if ( strcmp(subcmd, "has_syms") == 0 ) {
        result = flags & HAS_SYMS;
    }
    else {
        Tcl_SetObjResult(interp, Tcl_NewStringObj(SUBCMDS,
                                                  strlen(SUBCMDS)));
        return TCL_ERROR;
    }

    result = (result != 0);

    Tcl_SetObjResult(interp, Tcl_NewIntObj(result));
    return TCL_OK;

#else /* ! BFD_ENABLED */

    Tcl_SetObjResult(interp, Tcl_NewStringObj(PSBFD_ENOTINSTALLED,                                               strlen(PSBFD_ENOTINSTALLED)));
    return TCL_ERROR;

#endif

}
