/* $Id: cxxfilt_init.c,v 1.11 2011/09/15 21:55:56 ruiliu Exp $ */

/****************************************************************************
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *      Copyright(C) 2003-2006, 2011, The Board of Trustees of the
 *             University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications 
 *              University of Illinois at Urbana-Champaign
 * 
 *                  http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * cxxfilt_init.c: Tcl interface to cplus_demangle
 *
 * Author: Rick Kufrin
 * Date:   5/23/2003
 *
 * This file is part of PerfSuite.
 *
 */

#include "cxxfilt.h"

int
Pscxxfilt_Init(Tcl_Interp *interp)
{
    if ( Tcl_InitStubs(interp, TCL_REQUIRED_VERSION, 0) == NULL ) {
        return TCL_ERROR;
    }

    Tcl_CreateObjCommand(interp, "::pscxxfilt::demangle",
                         cxxfilt_demangleObjCmd,
                         (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

    Tcl_PkgProvide(interp, PSCXXFILT_PACKAGE_NAME, PSCXXFILT_PACKAGE_VERSION);

    return TCL_OK;
}

/*
 * Single procedure in this package for demangling strings
 * provided as an argument.
 *
 * ::pscxxfilt::demangle ?option? string
 *
 * where option is yet-to-be-implemented.
 *
 */

int
cxxfilt_demangleObjCmd(ClientData clientData, Tcl_Interp *interp,
                       int objc, Tcl_Obj *CONST objv[])
{
    char *option, *mangled, *demangled;
    int index = 1;

    if ( (objc < 2) || (objc > 3) ) {
        Tcl_WrongNumArgs(interp, 1, objv, "?option-style? string");
        return TCL_ERROR;
    }

    if ( objc == 3 ) {
        option = Tcl_GetString(objv[1]);
        if (option[0] != '-') {
            Tcl_SetObjResult(interp,
                              Tcl_NewStringObj(PSCXXFILT_BADOPTION,
                                               strlen(PSCXXFILT_BADOPTION)));
            return TCL_ERROR;
        }
        option++;
        index++;
    }

    mangled = Tcl_GetString(objv[index]);

#if ! CPLUS_DEMANGLE_PIC
    /*
     * If we don't have support for cplus_demangle(), just return
     * the original name.
     */
    demangled = (char *) 0;
#elif defined(DMGL_PARAMS) && defined(DMGL_ANSI)
    demangled = cplus_demangle(mangled, DMGL_PARAMS|DMGL_ANSI);
#else
    demangled = cplus_demangle(mangled, 1);
#endif

    if ( ! demangled ) {
        demangled = mangled;
    }

    Tcl_SetObjResult(interp, Tcl_NewStringObj(demangled, strlen(demangled)));

    return TCL_OK;
}
