/* $Id: psmd5.c,v 1.1 2006/10/02 14:06:01 rkufrin Exp $ */

/****************************************************************************
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *           Copyright(C) 2006, The Board of Trustees of the
 *             University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications 
 *              University of Illinois at Urbana-Champaign
 * 
 *                  http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

/*
 * psmd5.c: Tcl interface to MD5 (via libperfsuite)
 *
 * Author: Rick Kufrin
 * Date:   10/1/2006
 *
 * This file is part of PerfSuite.
 *
 */

#include <string.h>
#include <errno.h>
#include <perfsuite.h>
#include <psinternal.h>
#include "psmd5.h"

int
Psmd_Init(Tcl_Interp *interp)
{
    if ( Tcl_InitStubs(interp, TCL_REQUIRED_VERSION, 0) == NULL ) {
        return TCL_ERROR;
    }

    Tcl_CreateObjCommand(interp, "::psmd5::md5",
                         md5ObjCmd,
                         (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

    Tcl_PkgProvide(interp, PSMD5_PACKAGE_NAME, PSMD5_PACKAGE_VERSION);

    return TCL_OK;
}

/*
 * Single procedure in this package for demangling strings
 * provided as an argument.
 *
 * ::psmd5::md5 ?option? argument
 *
 */

int
md5ObjCmd(ClientData clientData, Tcl_Interp *interp,
          int objc, Tcl_Obj *CONST objv[])
{
    char *option, *filename, md5out[PS_MD5LEN];
    const char *errstring;
    int ret;

    if ( objc != 3 ) {
        Tcl_WrongNumArgs(interp, 1, objv, "-file argument");
        return TCL_ERROR;
    }

    option = Tcl_GetString(objv[1]);
    if ( strcmp(option, "-file") != 0 ) {
        Tcl_SetObjResult(interp,
                          Tcl_NewStringObj(PSMD5_BADOPTION,
                                           strlen(PSMD5_BADOPTION)));
        return TCL_ERROR;
    }

    filename = Tcl_GetString(objv[2]);

    ret =_ps_md5_file(filename, md5out);
    if ( ret != PS_SUCCESS ) {
        if ( ret == PS_ESYS ) {
            errstring = strerror(errno);
        }
        else {
            errstring = ps_strerror(ret);
        }
        Tcl_SetObjResult(interp,
                         Tcl_NewStringObj(errstring, strlen(errstring)));
        return TCL_ERROR;
    }

    Tcl_SetObjResult(interp, Tcl_NewStringObj(md5out, PS_MD5LEN-1));

    return TCL_OK;
}
