/* $Id: info.c,v 1.21 2013/01/30 21:13:05 ruiliu Exp $ */

/****************************************************************************
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *     Copyright(C) 2002-2006, 2013, The Board of Trustees of the
 *            University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                  http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/
/*
 * info.c: information output routines for psrun
 *
 * Author: Rick Kufrin
 * Date:   12/14/2002
 *
 * This file is part of PerfSuite.
 */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>

#include <hwpc.h>

#include "psrun.h"
#include "resource.h"
#include "info.h"

extern char * PS_HWPC_DEF_CONFIGFILE;

void
banner(void)
{
    printf("%s (version %s) performance measurement tool\n",
           PACKAGE_NAME, PACKAGE_VERSION);
    return;
}

void
info(void)
{
    int supported;

    banner();

    puts("\nThis installation of psrun supports the following capabilities:\n");

    fputs("\tProfiling:\t", stdout);
    supported = 0;
#if HAVE_SETITIMER
    fputs(" itimer", stdout);
    supported++;
#endif
#if HAVE_PAPI
    fputs(" PAPI", stdout);
    supported++;
#endif
#if HAVE_PERFMON
    fputs(" Perfmon", stdout);
    supported++;
#endif
#if HAVE_PERFMON2
    fputs(" Perfmon2", stdout);
    supported++;
#endif
#if HAVE_PROFIL
    fputs(" profil", stdout);
    supported++;
#endif
    if ( ! supported ) {
        fputs(" (not supported)", stdout);
    }
    puts("");

    fputs("\tCounting:\t", stdout);
    supported = 0;
#if HAVE_PAPI
    fputs(" PAPI", stdout);
    supported++;
#endif
#if HAVE_PERFMON
    fputs(" Perfmon", stdout);
    supported++;
#endif
#if HAVE_PERFMON2
    fputs(" Perfmon2", stdout);
    supported++;
#endif
    if ( ! supported ) {
        fputs(" (not supported)", stdout);
    }
    puts("");

    fputs("\tMD5 checksums:\t", stdout);
#if HAVE_MD5_RFC
    puts(" yes");
#else
    puts(" no");
#endif

    fputs("\tResource usage:\t", stdout);
    puts(" yes");

    showconfig();

    return;
}

void
showconfig(void)
{
    int ret;
    fputs("\nStandard configuration file directory for ", stdout);
    puts("psrun on this computer:\n");

    printf("\t%s%s\n", PS_DATADIR, "/xml/pshwpc");

    ret = set_def_config();
    if (ret != PS_SUCCESS) {
        printf ("set_def_config() returned error code: %d.\n", ret);
    }
    printf("\nDefault configuration file for this computer: %s\n\n",
           PS_HWPC_DEF_CONFIGFILE);

    return;
}

void
version(void)
{
    puts  ("\t--------------------------------------------");
    printf("\t               %s\n", PACKAGE_STRING);
    printf("\t                  psrun %s\n", PROGRAM_VERSION);
    puts  ("");
    puts  ("\t         University of Illinois/NCSA");
    puts  ("\t             Open Source License");
    puts  ("");
    /*printf("\tEmail: %s\n", PACKAGE_BUGREPORT);*/
    puts  ("\t        http://perfsuite.ncsa.uiuc.edu/");
    puts  ("\t       http://perfsuite.sourceforge.net/");
    puts  ("\t--------------------------------------------");

    return;
}
