/* $Id: libpsrun.h,v 1.19 2013/01/29 22:12:34 ruiliu Exp $ */

/****************************************************************************
 *
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *       Copyright(C) 2004-2010, 2013, The Board of Trustees of the
 *              University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *            National Center for Supercomputing Applications
 *              University of Illinois at Urbana-Champaign
 * 
 *                   http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

#ifndef LIBPSRUN_H_INCLUDED
#define LIBPSRUN_H_INCLUDED 1

#if _REENTRANT
# if HAVE_PTHREAD
#  include <pthread.h>
# endif
#endif

#include <perfsuite.h>

#ifndef LIBPSRUN_NON_EXTERN
/*
 * Flag to indicated whether psrun_init() has been called.
 */
extern int _psrun_initialized;

/*
 * These control handling of errors on startup
 */
#define PSRUN_SIGPROF_IGN 1
extern int _psrun_ignoreerrors;
#endif  /* LIBPSRUN_NON_EXTERN */

/*
 * This controls handling of problems on process exit
 */
#define BLOCKERRORSONEXIT 1

/* @@ RTLD_NOW or RTLD_LAZY? */
#define DLRESOLVE_MODE RTLD_LAZY

#define CHECK_DLSYM \
    if ((error = dlerror()) != NULL) { \
        fprintf(stderr, "%s\n", error); \
        exit(1); \
    }

/*
 * Routines that will need to be called in the
 * hwpc shared lib
 */
#define PS_HWPC_INIT              "ps_hwpc_init"
#define PS_HWPC_START             "ps_hwpc_start"
#define PS_HWPC_STOP              "ps_hwpc_stop"
#define PS_HWPC_SHUTDOWN          "ps_hwpc_shutdown"

#define FORK                      "fork"

/*
 * Pthreads library and entry points that will be needed 
 */
#define PTHREAD_SHLIB             "libpthread.so.0"
#define PTHREAD_CREATE            "pthread_create"

BEGIN_C_DECLS

#ifndef LIBPSRUN_NON_EXTERN
extern int (*_psrun_hwpc_init)     PARAMS((void));
extern int (*_psrun_hwpc_start)    PARAMS((void));
extern int (*_psrun_hwpc_stop)     PARAMS((char *));
extern int (*_psrun_hwpc_shutdown) PARAMS((void));

extern pid_t (*_psrun_real_fork)   PARAMS((void));

#if _REENTRANT
extern int (*_psrun_real_pthread_create) PARAMS(
                                  (pthread_t * restrict thread,
				   const pthread_attr_t * restrict attr,
				   void *(*start_routine) (void *),
				   void * restrict arg));
#endif
#endif  /* LIBPSRUN_NON_EXTERN */

END_C_DECLS

#endif /* LIBPSRUN_H_INCLUDED */
