/* $Id: resource.h,v 1.29 2012/05/11 17:11:00 ruiliu Exp $ */

/****************************************************************************
 *                    University of Illinois/NCSA
 *                        Open Source License
 * 
 *     Copyright(C) 2002-2006, 2012, The Board of Trustees of the
 *            University of Illinois. All rights reserved.
 * 
 *                            Developed by:             
 * 
 *                        The PerfSuite Project
 *           National Center for Supercomputing Applications
 *             University of Illinois at Urbana-Champaign
 * 
 *                  http://perfsuite.ncsa.uiuc.edu/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * + Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimers.
 * + Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in
 *   the documentation and/or other materials provided with the distribution.
 * + Neither the names of The PerfSuite Project, NCSA/University of Illinois
 *   at Urbana-Champaign, nor the names of its contributors may be used to
 *   endorse or promote products derived from this Software without specific
 *   prior written permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS WITH THE SOFTWARE.
 ****************************************************************************/

#ifndef _PSRUN_RESOURCE_H_INCLUDED
#define _PSRUN_RESOURCE_H_INCLUDED 1

#if HAVE_CONFIG_H
# include <config.h>
#endif

#if HAVE_SYS_TIME_H
# include <sys/time.h>
#endif

#if HAVE_SYS_RESOURCE_H
# include <sys/resource.h>
#endif

#if HAVE_UNISTD_H
# include <unistd.h>
#endif

#include <perfsuite.h>

#define PSRUN_RESOURCE_VERSION "0.2"
#define PSRUN_RESOURCE_SUFFIX  "res"

#define INDENT 1
#define INDENT_CHAR '\t'

/*
 * Default amount of time (seconds) to wait
 * in between checks of resource usage.
 */
#define PSRUN_DEFAULT_SLEEPSECS 1

#define PSRUN_DEFAULTLOADBINS 1000

enum {LOADAVG1MIN=0,
      LOADAVG5MIN=1,
      LOADAVG15MIN=2 } loadavg_t;

/*
 * For the psrun executable itself (resource tracking by monitor thread)
 */
typedef struct {
    unsigned int  count;        /* Count of samples taken               */
    int           sleepsecs;    /* Seconds to sleep in between samples  */
    pid_t         pid;          /* PID we're monitoring                 */
    unsigned int  maxrss;       /* Maximum resident set size            */
    unsigned int  maxvsize;     /* Maximum virtual memory size          */
    int           status;       /* Exit status of the monitored process */
    int           code;         /* Exit code or signal causing exit     */
    struct rusage rusage;       /* getrusage() return info              */
    ps_procstat_t procstat;     /* Process information                  */
    double        loadavg[3];   /* 1-min load avg high-water mark       */
    double        loadmax[3];   /* 1, 5, 15-min high-water marks        */
    int           *loadhist;    /* Histogram of load averages over run  */
    int           loadbins;     /* Number of bins in the histogram      */
} psrun_resinfo_t;

#ifndef MEGABYTE
#define MEGABYTE 1045876.0
#endif

/* XML Element Tags */

#define RESOURCEREPORTTAG "resourcereport"
#define COMMANDTAG        "command"
#define ANNOTATIONTAG     "annotation"
#define PIDTAG            "pid"
#define STARTTIMETAG      "starttime"
#define EXITCODETAG       "exitcode"
#define SAMPLESTAG        "samples"
#define LOADAVGSTAG       "loadavgs"
#define LOADAVGTAG        "loadavg"
#define MAXLOADSTAG       "maxloads"
#define MAXLOADTAG        "maxload"
#define MEMORYUSETAG      "memoryuse"
#define MAXRSSTAG         "maxrss"
#define MAXVMTAG          "maxvm"
#define CPUTIMETAG        "cputime"
#define USERTIMETAG       "usertime"
#define SYSTEMTIMETAG     "systemtime"
#define FAULTSTAG         "faults"
#define MAJORTAG          "major"
#define MINORTAG          "minor"
#define SWAPSTAG          "swaps"

/* Used in text output */

#define MAJORSEPLENGTH    78
#define MAJORSEPCHAR      '='
#define LABELSEPPOS      (30 - 1)
#define LABELSEPCHAR     ':'

extern int (*psrun_resource_write)(psrun_resinfo_t *);

BEGIN_C_DECLS
extern int block_signals             PARAMS((void));
extern void *monitor_resource        PARAMS((void *resource_arg));
extern int psrun_resource_text_write PARAMS((psrun_resinfo_t *res));
extern int psrun_resource_xml_write  PARAMS((psrun_resinfo_t *res));
extern int resource_init             PARAMS((pthread_t *monitor_thread,
                                             psrun_resinfo_t *res));
extern int resource_start            PARAMS((void));
extern int resource_stop             PARAMS((pthread_t *monitor_thread,
                                             psrun_resinfo_t *res));
END_C_DECLS

#endif /* _PSRUN_RESOURCE_H_INCLUDED */
